# -*- coding: utf-8 -*-
"""
    =====================
    @ kyaah software
    =====================

      / / \/ /|   /|   /  /
     /_/  / /_|  /_|  /__/
    /  / / /  | /  | /  /

    Kyaah abstract away cognitive over-head of sending SMTP mail, together with other mailing operations things like, mail with file, tokens etc.

    Usage at a glance:
        >>> import kyaah

        >>> sender = 'my_email@gmail.com'
        >>> receiver = ['receiver_1@gmail.com', 'receiver_2@gmail.com']
        >>> passwd = '*********' # use app password

        >>> server = 'gmail'
        >>> subj = f'Hellow world!'
        >>> body = 'Lorem ipsum dolor sit amet adipisicing elit, rerum voluptate ipsum volupt.'
        
        >>> kyaah.sendMail(from_usr=sender, to_usr=receiver, svr=server, subject=subj, body=body, mail_passwd=passwd)
"""

from datetime import datetime

__title__ = 'kyaah'
__version__ = '0.1.11'
__author__ = 'Usman Musa'
__author_email__ = 'usmanmusa1920@gmail.com'
__author_website__ = 'https://usmanmusa1920.github.io'
__repository__ = 'https://github.com/usmanmusa1920/kyaah'
__url__ = 'https://kyaah.readthedocs.io'
__license__ = 'MIT'
__copyright__ = f'Copyright (C) 2022 - {datetime.today().year} Usman Musa'
__description__ = 'Kyaah abstract away cognitive over-head of sending SMTP mail, together with other mailing operations things like, mail with file, tokens etc.'


status_codes = {
    # Information.
    200: ('success',),
    # Client Error.
    400: ('bad_request',),
    401: ('unauthorized',),
    404: ('not_found',),
    # Server Error.
    500: ('internal_server_error',),
    503: ('service_unavailable',),
}


def selector(env=None):
    """decide a class for `environment variable` for security purpose or not"""
    if env:
        cls = Vault
    else:
        cls = BaseMail
    return cls


from .base import BaseMail
from .vault import Vault

from .utils import Tokens
from .utils import Faker
from .server import Serve

from .api import localMail
from .api import sendMail
from .api import sendImages
from .api import sendFiles
from .api import sendPage
from .api import fk
from .api import otp



# -*- coding: utf-8 -*-
import json
import math
import random
import geocoder
import requests


class Tokens:
    """The base class for OTP mail"""
    
    def mail_otp(self, _range=6):
        """send otp code"""
        digits = '0123456789'
        OTP = ''
        for _ in range(_range):
            OTP += digits[math.floor(random.random()*10)]
        return OTP
    

class Faker:
    """class that give you random email address"""
    
    _source = ['https://www.guerrillamail.com/ajax.php?f=get_email_address&ip=', '&agent=Mozilla_foo_bar']
    
    def __init__(self, url=_source, ip='me'):
        self.url = url
        self.ip = ip

    def what_ip(self):
        """find ip address of the requester"""
        g = geocoder.ip(self.ip).raw
        ip = g.get('ip')
        return ip
    
    @property
    def make_request(self):
        """
        This method make a request along side with an ip address from `what_ip` method of this class
        """
        url_first = self.url[0]
        url_mid = self.url[1]
        url_end = self.what_ip()
        
        com_url = url_first + url_end + url_mid
        r = requests.get(com_url).text
        return r
    
    def faker(self) -> dict:
        """give the actual maill address generated from guerrillamail"""
        gue_raw = json.loads(self.make_request)
        _fake = gue_raw['email_addr']
        return {'fake_email': _fake}



# -*- coding: utf-8 -*-
server = {
    'gmail': {
        'server': ['smtp.gmail.com'],
        'port': [465]
    },
    
    'local': {
        'server': ['localhost'],
        'port': [1025]
    },
    
    'yahoo': {
        'server': ['smtp.mail.yahoo.com', 'pop.mail.yahoo.com', 'imap.mail.yahoo.com'],
        'port': [465, 587, 995, 993]
    }
}

"""
port 995 is for yahoo pop server
port 993 is for yahoo imap server
"""

class Serve:
    """this class handle SMTP server suffs"""
    def mail(mail):
        return server[mail]



# -*- coding: utf-8 -*-
import os
from . import BaseMail


class Vault(BaseMail):
    """
    This class handle environment variable information
    
    use this class if you want to access your `mail address` and `password/app_password` from environment variable, all what you need is to put your `mail address environment variable` in place of your email address, likewise the password/app_password
    """
    
    @property
    def from_usr_addr(self):
        """get sender mail address from environment variable"""
        return os.environ.get(self.from_usr)
    
    @property
    def mail_passwd_env(self):
        """get sender mail password/app_password from environment variable"""
        return os.environ.get(self.mail_passwd)



# -*- coding: utf-8 -*-
import logging

from . import Serve
from . import Faker
from . import Tokens
from . import selector


def send():
    """
    This function can be use for any kind of mail operation, being it,
    simple mail, attach file for a mail, attach html page with mail,
    attach image file, etc
    """
    pass


def localMail(svr=None, env=False, port=False, **kwargs):
    r"""
    To test your mail locally, first boot up the python mail server on a different terminal by:
        `python3 -m smtpd -c DebuggingServer -n localhost:1025`
    and then open another terminal and run your python script including the below code in it:

    Usage:
        >>> import kyaah
        >>> sender = 'youremail@gmail.com'
        >>> receiver = ['receiver_1@gmail.com', 'receiver_2@gmail.com']
        >>> passwd = '*********' # use app password
        >>> mail_serve = 'local'
        >>> subj = f'Hellow world!'
        >>> body = 'Lorem ipsum dolor sit amet adipisicing elit, rerum voluptate ipsum volupt.'
        
        >>> kyaah.localMail(from_usr=sender, to_usr=receiver, svr=mail_serve, subject=subj, body=body, mail_passwd=passwd)
    """

    s_mail = Serve.mail(svr)
    cls = selector(env=env)
    base = cls(
        # slicing the first index of the server list, even though it's only one item
        server = s_mail['server'][0], **kwargs
        )
    # slicing the first index of the port list, even though it's only one item
    if port:
        base.local_mail(port=port)
    else:
        base.local_mail()


def sendMail(svr=None, env=False, **kwargs):
    r"""
    send a simple SMTP mail
    :svr: this is the type of the sender mail, for google -> gmail, for yahoo -> yahoo, etc

    Usage:
        >>> import kyaah
        >>> sender = 'youremail@gmail.com'
        >>> receiver = ['receiver_1@gmail.com', 'receiver_2@gmail.com']
        >>> passwd = '*********' # use app password
        >>> mail_serve = 'gmail'
        >>> subj = f'Hellow world!'
        >>> body = 'Lorem ipsum dolor sit amet adipisicing elit, rerum voluptate ipsum volupt.'

        >>> kyaah.sendMail(from_usr=sender, to_usr=receiver, svr=mail_serve, subject=subj, body=body, mail_passwd=passwd)
    """
    
    s_mail = Serve.mail(svr)
    cls = selector(env=env)
    base = cls(
        # slicing the first index of the server list, even though it's only one item
        server = s_mail['server'][0], **kwargs
        )
    # slicing the first index of the port list, even though it's only one item
    base.send_mail(port=s_mail['port'][0])


def sendImages(images=None, env=False, svr=None, **kwargs):
    r"""
    send simple mail with image(s)

    Usage:
        >>> import kyaah
        >>> sender = 'youremail@gmail.com'
        >>> receiver = ['receiver_1@gmail.com', 'receiver_2@gmail.com']
        >>> passwd = '*********' # use app password
        >>> mail_serve = 'gmail'
        >>> subj = f'Hellow world!'
        >>> body = 'Lorem ipsum dolor sit amet adipisicing elit, rerum voluptate ipsum volupt.'
        >>> images='my_image.jpg'

        >>> kyaah.sendImages(images=images, from_usr=sender, to_usr=receiver, svr=mail_serve, subject=subj, body=body, mail_passwd=passwd)
    """
    s_mail = Serve.mail(svr)
    cls = selector(env=env)
    base = cls(
        # slicing the first index of the server list, even though it's only one item
        server = s_mail['server'][0], **kwargs
        )
    # slicing the first index of the port list, even though it's only one item
    base.mail_with_image(images, port=s_mail['port'][0])


def sendFiles(files=None, env=False, svr=None, **kwargs):
    r"""
    send simple mail with file(s)

    Usage:
        >>> import kyaah
        >>> sender = 'youremail@gmail.com'
        >>> receiver = ['receiver_1@gmail.com', 'receiver_2@gmail.com']
        >>> passwd = '*********' # use app password
        >>> mail_serve = 'gmail'
        >>> subj = f'Hellow world!'
        >>> body = 'Lorem ipsum dolor sit amet adipisicing elit, rerum voluptate ipsum volupt.'
        >>> files=['em.py', 'test.py', '/home/usman/Desktop/media/Usman.jpg']

        >>> kyaah.sendFiles(files=files, from_usr=sender, to_usr=receiver, svr=mail_serve, subject=subj, body=body, mail_passwd=passwd)
    """
    s_mail = Serve.mail(svr)
    cls = selector(env=env)
    base = cls(
        # slicing the first index of the server list, even though it's only one item
        server = s_mail['server'][0], **kwargs
        )
    # slicing the first index of the port list, even though it's only one item
    base.mail_with_file(files, port=s_mail['port'][0])


def sendPage(page='default', env=False, svr=None, **kwargs):
    r"""
    send simple mail with page
    
    Usage:
        >>> import kyaah
        >>> sender = 'youremail@gmail.com'
        >>> receiver = ['receiver_1@gmail.com', 'receiver_2@gmail.com']
        >>> passwd = '*********' # use app password
        >>> mail_serve = 'gmail'
        >>> subj = f'Hellow world!'
        >>> body = 'Lorem ipsum dolor sit amet adipisicing elit, rerum voluptate ipsum volupt.'
        
        >>> kyaah.sendPage(page='index.html', from_usr=sender, to_usr=receiver, svr=mail_serve, subject=subj, body=body, mail_passwd=passwd)
    """
    s_mail = Serve.mail(svr)
    cls = selector(env=env)
    base = cls(
        # slicing the first index of the server list, even though it's only one item
        server = s_mail['server'][0], **kwargs
        )
    # slicing the first index of the port list, even though it's only one item
    base.mail_with_page(file=page, port=s_mail['port'][0])


def fk():
    """This function generate a random email address for you"""
    formatter = '[+] [%(asctime)s] [%(levelname)s] %(message)s'
    logging.basicConfig(format = formatter)
    logger = logging.getLogger()
    logger.setLevel(logging.DEBUG)
    logger.info(Faker().faker())


def otp(_range=False):
    """This function give you a random OTP code"""
    if _range:
        return Tokens().mail_otp(_range)
    return Tokens().mail_otp()



# -*- coding: utf-8 -*-
import re
import os
from setuptools import setup
from setuptools import find_packages


def grep(attrname):
    """get package info from __init__.py file"""
    file_path = os.path.join(os.path.dirname(__file__), 'kyaah/__init__.py')

    # content of the file
    read_file = open(file_path).read()

    # pattern match using regex
    pattern = r"{0}\W*=\W*'([^']+)'".format(attrname)

    # our regex value
    attr_value, = re.findall(pattern, read_file)
    return attr_value


setup(
    name = grep('__title__'), # name of the main package (base folder i.e kyaah)
    version = grep('__version__'),
    description = grep('__description__'),
    long_description = open('README.md').read() + '\n\n' + open('CHANGELOG').read(),
    long_description_content_type='text/markdown',
    python_requires = '>=3.7',
    platforms='any',
    
    url = grep('__url__'),
    download_url = 'https://pypi.org/project/kyaah',
    author = grep('__author__'),
    author_email = grep('__author_email__'),
    license = grep('__license__'),
    classifiers = [
        'Development Status :: 5 - Production/Stable',
        'Environment :: Other Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Topic :: Internet',
        'Topic :: Communications :: Email',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    
    # used when people are searching for a module, keywords separated with a space
    keywords = 'kyaah',
    include_package_data = True, # include files listed in MANIFEST.in
    
    # The list of packages(directories) for your library
    packages = find_packages(), # OR packages=['kyaah'] 
    # If your package is a single module, use this instead of 'packages':
    # py_modules=[''] # list of files (modules) that are not in any directory (at the root dir)
    # the libraries it depends on
    
    # List of other python modules which this module depends on.  For example RPi.GPIO
    install_requires = [
        'geocoder>=1.38.1',
    ],
    project_urls={
        'Documentation': grep('__url__'),
        'Source': grep('__repository__'),
    },
)



# -*- coding: utf-8 -*-
import imghdr
import logging
import smtplib
from email.message import EmailMessage

"""
  NOTSET    ---  0
  DEBUG     ---  10
  INFO      ---  20
  WARNING   ---  30  (default)
  ERROR     ---  40
  CRITICAL  ---  50
"""

FORMATTER = '[+] [%(asctime)s] [%(levelname)s] %(message)s'
logging.basicConfig(format=FORMATTER)
LOGGER = logging.getLogger()  # Creating an object
LOGGER.setLevel(logging.DEBUG)  # Setting the threshold of LOGGER to DEBUG


class BaseMail:
    """
    The base class for mail
    Note:
      in all `BaseMail` mathod there is a keyword argument called `port`, that port is the port of your SMTP server which will listen, make sure to put the port that is convenient to your SMTP server e.g for gmail is 465, although kyaah has a dict tionary of server, where each server has a list of ports, SMTP server, etc.
    """

    def __init__(
        self,
        from_usr=None,
        to_usr=None,
        subject=None,
        body=None,
        mail_passwd=None,
        server=None,
    ):

        self.from_usr = from_usr
        self.to_usr = to_usr
        self.subject = subject
        self.body = body
        self.mail_passwd = mail_passwd
        self.server = server

        while self.from_usr == None:
            raise UnboundLocalError(f'`from_usr` is required')
        while self.to_usr == None:
            raise UnboundLocalError(f'`to_usr` is required')

    @property
    def from_usr_addr(self):
        """the sender mail address"""
        return self.from_usr

    @property
    def mail_passwd_env(self):
        """the sender mail password"""
        return self.mail_passwd

    @property
    def mail_msg(self):
        """content of the mail (subject and message body)"""
        subj = self.subject
        body = self.body
        data = f'Subject: {subj}\n\n{body}'
        return data

    def check_required(self):
        """checking for required values"""
        while self.mail_passwd == None:
            raise UnboundLocalError(f'`mail_passwd` is required')
        while self.server == None:
            raise UnboundLocalError(f'`server` is required')

    def mail_head(self, msg):
        """contains the sender, receiver and subject"""
        # grab the subject from `mail_msg` method by spliting it,
        # and grab the first index which is the subject,
        # then we slice the first index from index `9` that mean we negate
        # `subj` variable of mail_msg method of this class
        # which is `Subject: `
        msg['Subject'] = self.mail_msg.split('\n')[0][9:]
        msg['From'] = self.from_usr_addr
        msg['To'] = self.to_usr

    def mail_open(self, msg, port):
        """
        This open the connection, and login to email address using email address provided together with password/app_password
        """
        with smtplib.SMTP_SSL(self.server, port) as smtp:
            smtp.login(self.from_usr_addr, self.mail_passwd_env)
            smtp.send_message(msg)

    def local_mail(self, port=1025):
        """localhost mail"""
        with smtplib.SMTP(self.server, port) as smtp:
            smtp.sendmail(self.from_usr_addr, self.to_usr, self.mail_msg)

    def send_mail(self, port=25):
        """send mail with ssl security"""
        # recepients is the list of receivers email address that you will send emaill address to, if the recepients is not included when call the method it will only send it to the `self.to_usr` that you pass in as the argument when instanciating the `BaseMail` class
        self.check_required()

        if len(self.to_usr) > 1:
            # f there is recepients
            for receiver in self.to_usr:
                with smtplib.SMTP_SSL(self.server, port) as smtp:
                    smtp.login(self.from_usr_addr, self.mail_passwd_env)
                    smtp.sendmail(self.from_usr_addr, receiver, self.mail_msg)
        elif len(self.to_usr) == 1:
            with smtplib.SMTP_SSL(self.server, port) as smtp:
                smtp.login(self.from_usr_addr, self.mail_passwd_env)
                smtp.sendmail(self.from_usr_addr, self.to_usr, self.mail_msg)

    def file_to_send(self, file_to, f_opration, _msg=None, maintype=None):
        # maintype = 'image' for sending image
        # maintype = 'application' for sending files like pdf

        if maintype == 'application':
            with open(file_to, f_opration) as file_send:
                f_read = file_send.read()
                f_name = file_send.name
            _msg.add_attachment(
                f_read, maintype=maintype, subtype='octet-stream', filename=f_name)
        if maintype == 'image':
            with open(file_to, f_opration) as img_send:
                img_read = img_send.read()
                f_type = imghdr.what(img_send.name)
                f_name = img_send.name
            _msg.add_attachment(
                img_read, maintype=maintype, subtype=f_type, filename=f_name)

    def mail_with_image(self, image, port=25):
        """send mail together with an image"""
        self.check_required()

        if len(self.to_usr) > 1:
            # if the list of to_usr is more than one
            for receiver in self.to_usr:
                msg = EmailMessage()
                msg['Subject'] = self.mail_msg.split('\n')[0][9:]
                msg['From'] = self.from_usr_addr
                msg['To'] = receiver
                msg.set_content(self.mail_msg.split('\n')[2])

                if type(image) == list:
                    # send message with more than one image
                    for my_img in image:
                        self.file_to_send(
                            my_img, 'rb', _msg=msg, maintype='image')
                    self.mail_open(msg, port)
                elif type(image) == str:
                    # send message with one image
                    self.file_to_send(image, 'rb', _msg=msg, maintype='image')
                    self.mail_open(msg, port)
                else:
                    LOGGER.error(
                        f'only list or string can be pass as argument in {self.mail_with_image}')
        elif len(self.to_usr) == 1:
            msg = EmailMessage()
            self.mail_head(msg)
            msg.set_content(self.mail_msg.split('\n')[2])

            if type(image) == list:
                # send message with more than one image, for only one destination
                for my_img in image:
                    self.file_to_send(my_img, 'rb', _msg=msg, maintype='image')
                self.mail_open(msg, port)
            elif type(image) == str:
                # send message with one image, for only one destination
                self.file_to_send(image, 'rb', _msg=msg, maintype='image')
                self.mail_open(msg, port)
            else:
                LOGGER.error(
                    f'only list or string can be pass as argument in {self.mail_with_image}')

    def mail_with_file(self, mail_files, port=25):
        """send mail together with a list of mail_files"""
        self.check_required()

        msg = EmailMessage()
        self.mail_head(msg)
        msg.set_content(self.mail_msg.split('\n')[2])

        for a_file in mail_files:
            self.file_to_send(a_file, 'rb', _msg=msg, maintype='application')
        self.mail_open(msg, port)

    def mail_with_page(self, file=None, port=25):
        """
        send mail with html page. You can pass a keyword args of `file='default'` which will send a default html page provided within this class method, that is for test

        But if you want to send yor own html page assign the `file` with a value of the absolute location of your html page
        """

        self.check_required()

        msg = EmailMessage()
        self.mail_head(msg)
        msg.set_content(self.mail_msg.split('\n')[2])

        if file == 'default':
            dummy_page = f'''<!DOCTYPE html>
<html lang="en">
<head>
  <meta charset="UTF-8">
  <meta http-equiv="X-UA-Compatible" content="IE=edge">
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
  <title>Document</title>
</head>
<body>
  <h1 style="color:SlateGray;">Hello world mail from {self.from_usr_addr}</h1>
  <p style="color:brown;">Lorem ipsum dolor sit amet consectetur adipisicing elit. Rerum voluptate ipsum voluptatum doloribus, incidunt totam doloremque quae quibusdam exercitationem sapiente vel veritatis consequuntur earum et. Quod nobis minus minima repellat.</p>
</body>
</html>
'''
            msg.add_alternative(dummy_page, subtype='html')
        elif file != None:
            with open(file) as f_html:
                html_f = f_html.read()
            msg.add_attachment(html_f, subtype='html')
        self.mail_open(msg, port)
